@@ -4,7 +4,7 @@
  *  
  *  (C) Daniel Sundberg 2000
  *  dss@home.se
- *  http://sumpan.campus.luth.se/software/
+ *  http://wmmultipop3.sf.net
  *  
  *  Based on:
  *  wmpop3.c
@@ -23,6 +23,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <time.h>
+#include <signal.h>
 
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -52,7 +53,7 @@ int  wminet_mask_height = 64;
 #define SEC_IN_MIN  60
 #define YES          1
 #define NO           0
-#define STR_LENGTH  32
+#define STR_LENGTH 128 
 #define MAX_RUN_COMMAND_STR_LENGTH      255
 
 char *ProgName;
@@ -71,6 +72,7 @@ typedef struct {
   MailServerConf **c;
   char mailclient[STR_LENGTH];
   char middle_click_cmd[STR_LENGTH];
+  char mail_arrived_cmd[STR_LENGTH];  /* Added by Roeland Huys 18/5/2002 */
   char right_click_cmd[STR_LENGTH];
   char run_command[MAX_RUN_COMMAND_STR_LENGTH];
   int  autoChecking;
@@ -102,8 +104,9 @@ int main(int argc, char *argv[]) {
   conf.debug = NO;
   conf.viewUnread=NO;
   ProgName = argv[0];
-  if (strlen(ProgName) >= 5)
+  if (strlen(ProgName) >= 5) {
     ProgName += (strlen(ProgName) - 5);
+  }
 	
   /* Parse command line arguments */
   for (i=1; i<argc; i++) {
@@ -136,13 +139,11 @@ int main(int argc, char *argv[]) {
 	if (conf.debug)
 	  printf("Viewing unread messages...\n");
 	break;
-
       case 'c' :
-	if (argc > (i+1))
-	  {
-	    strncpy(config_file, argv[i+1], sizeof(config_file));
-	    i++;
-	  }
+	if (argc > (i+1)) {
+	  strncpy(config_file, argv[i+1], sizeof(config_file));
+	  i++;
+	}
 	break;
       default:
 	usage();
@@ -151,9 +152,11 @@ int main(int argc, char *argv[]) {
       }
     }
   }
-  /* End of commandline parsing */
 
+  /* End of commandline parsing */
+  //  signal(SIGCHLD, SIG_IGN);
   wmCheckMail_routine(argc, argv); 
+  //  signal(SIGCHLD, SIG_DFL);
   /*readConfigFile(config_file);*/
   return 0;
 }
@@ -163,6 +166,7 @@ int main(int argc, char *argv[]) {
  *           for wmpop3 and check for mail every n number of minutes.
  */
 
+  static int nTotMail_last = 0;
 void wmCheckMail_routine(int argc, char **argv){
   int    nTotMail       = 0; 
   long   startTime      = 0; 
@@ -177,6 +181,8 @@ void wmCheckMail_routine(int argc, char **argv){
   Pop3   pc; 
   char   *display_name  = NULL;
   Display *temp_display  = NULL;
+  char command[1024];
+  int num_msgs, read_msgs;
   
   /* Find config-file */
   if( !strcmp( config_file, "not-defined") ) {
@@ -244,31 +250,57 @@ void wmCheckMail_routine(int argc, char **argv){
 	BlitString("Chk:     ", 5, (11*(4-1)) + 5); 
 	BlitString(conf.c[tempPopCnt]->mailserverid, 33, (11*(4-1)) + 5); 
 	RedrawWindow();
-	if (pop3MakeConnection(pc, conf.c[tempPopCnt]->popserver, conf.c[tempPopCnt]->port)==-1) {
-	  mailWaiting = -1;
-	  j = tempPopCnt;
-	} else if (pop3Login(pc, conf.c[tempPopCnt]->username,conf.c[tempPopCnt]->password) == -1) {
-	  mailWaiting = -1;
-	  j = tempPopCnt;
-	} else if ((pop3CheckMail(pc)) == -1) {
-	  mailWaiting = -1;
-	  j = tempPopCnt;
-	} else {
-	  /* Now we can check if we got new mail */
-	  conf.c[tempPopCnt]->nMsgs   = pop3GetNumberOfUnreadMessages(pc);
-	  conf.c[tempPopCnt]->nUnread = pop3GetTotalNumberOfMessages(pc);
-	  if (conf.viewUnread == YES) {
-	    printf("Adding unread msgs....\n");
-	    nTotMail = nTotMail + conf.c[tempPopCnt]->nUnread;
-	  } else {
-	    nTotMail = nTotMail + conf.c[tempPopCnt]->nMsgs;
+	
+	if (conf.c[tempPopCnt]->popserver[0] == '/') {
+	  snprintf( command, 1024, "%s%s%s", "exit `grep \"^From \" ",
+	            conf.c[tempPopCnt]->popserver, " | wc -l`" );
+	  num_msgs = WEXITSTATUS(system(command));
+	  if (num_msgs < 0) {
+	    num_msgs = 0;
 	  }
-	  if ((conf.c[tempPopCnt]->nMsgs > 0) || (conf.c[tempPopCnt]->nUnread > 0))
+	  conf.c[tempPopCnt]->nMsgs = num_msgs;
+	  snprintf( command, 1024, "%s%s%s", "exit `grep \"^Status: R\" ",
+	            conf.c[tempPopCnt]->popserver, " | wc -l`" );
+	  read_msgs = WEXITSTATUS(system(command));
+	  if (read_msgs < 0) { 
+	    read_msgs = 0;
+	  }
+	  if (read_msgs > num_msgs) {
+	    read_msgs = num_msgs;
+	  }
+	  conf.c[tempPopCnt]->nUnread = num_msgs - read_msgs;
+	  nTotMail += conf.c[tempPopCnt]->nUnread;
+	  if ((conf.c[tempPopCnt]->nMsgs > 0) || (conf.c[tempPopCnt]->nUnread > 0)) {
 	    j = tempPopCnt;
+	  }
+	} else {
+	  if (pop3MakeConnection(pc, conf.c[tempPopCnt]->popserver, conf.c[tempPopCnt]->port)==-1) {
+	    mailWaiting = -1;
+	    j = tempPopCnt;
+	  } else if (pop3Login(pc, conf.c[tempPopCnt]->username,conf.c[tempPopCnt]->password) == -1) {
+	    mailWaiting = -1;
+	    j = tempPopCnt;
+	  } else if ((pop3CheckMail(pc)) == -1) {
+	    mailWaiting = -1;
+	    j = tempPopCnt;
+	  } else {
+	    /* Now we can check if we got new mail */
+	    conf.c[tempPopCnt]->nMsgs   = pop3GetNumberOfUnreadMessages(pc);
+	    conf.c[tempPopCnt]->nUnread = pop3GetTotalNumberOfMessages(pc);
+	    if (conf.viewUnread == YES) {
+	      printf("Adding unread msgs....\n");
+	      nTotMail = nTotMail + conf.c[tempPopCnt]->nUnread;
+	    } else {
+	      nTotMail = nTotMail + conf.c[tempPopCnt]->nMsgs;
+	    }
+	    if ((conf.c[tempPopCnt]->nMsgs > 0) || (conf.c[tempPopCnt]->nUnread > 0))
+	      j = tempPopCnt;
+	  }
 	}
-	if (conf.debug == YES)
+	if (conf.debug == YES) {
 	  printf("Checking %s...%d/%d\n", conf.c[tempPopCnt]->popserver,
 		 conf.c[tempPopCnt]->nMsgs,conf.c[tempPopCnt]->nUnread); 
+	}
 	pop3Quit(pc);
       }
       
@@ -278,7 +310,7 @@ void wmCheckMail_routine(int argc, char **argv){
       nextCheckTime = time(0) + (conf.mailCheckDelay * SEC_IN_MIN); 
       
       /* Can someone tell me if or why I need this line? (This was a comment from the author 
-         of wmpop3). Seems to work on my system (dss@home.se) if you want this uncommented */
+         of wmpop3). Seems to work on my system, mail (dss@home.se) if you want this uncommented */
       /* waitpid(0, NULL, WNOHANG); */
 
       if( mailWaiting == -1 ){ 
@@ -286,15 +318,25 @@ void wmCheckMail_routine(int argc, char **argv){
         BlitString("Err:  ", 5, (11*(4-1)) + 5); 
         BlitString(conf.c[j]->mailserverid, 33, (11*(4-1)) + 5); 
       } 
-      if (conf.debug == YES)
+
+      /* Added by Roeland Huys 18/5/2002 */
+      if (( nTotMail > nTotMail_last) && strlen(conf.mail_arrived_cmd)) {
+	execCommand(conf.mail_arrived_cmd);
+      }
+      nTotMail_last = nTotMail;
+
+      if (conf.debug == YES) {
 	printf("%d new messages!\n", nTotMail);
+      }
+
       BlitString("   :  /", 5, (11*(4-1)) + 5); 
       BlitString(conf.c[j]->mailserverid, 5, (11*(4-1)) + 5); 
       printf("nUnread == %d, nMsgs == %d\n", conf.c[j]->nUnread, conf.c[j]->nMsgs);
-      if (conf.viewUnread == YES)
+      if (conf.viewUnread == YES) {
 	BlitNum(conf.c[j]->nUnread, 30, (11*(4-1)) + 5); 
-      else
+      }	else {
 	BlitNum(conf.c[j]->nMsgs, 30, (11*(4-1)) + 5); 
+      }
       BlitNum(nTotMail, 45, (11*(4-1)) + 5); 
 
       /*      printf("nTotMail == %d\n", nTotMail);*/
@@ -342,10 +384,11 @@ void wmCheckMail_routine(int argc, char **argv){
 	    }
 	    BlitString("   :  /", 4, (11*(4-1)) + 5); 
 	    BlitString(conf.c[j]->mailserverid, 5, (11*(4-1)) + 5); 
-	    if (conf.viewUnread == YES)
+	    if (conf.viewUnread == YES) {
 	      BlitNum(conf.c[j]->nUnread, 29, (11*(4-1)) + 5); 
-	    else
+	    } else {
 	      BlitNum(conf.c[j]->nMsgs, 29, (11*(4-1)) + 5); 
+	    }
   	    break; 
 	    BlitNum(nTotMail, 45, (11*(4-1)) + 5); 
 	  case 3:
@@ -400,11 +443,15 @@ void wmCheckMail_routine(int argc, char **argv){
 	buttonStatus = -1;
 	RedrawWindow();
 	break;
+      default:
+	//	printf("event type: %d\n", Event.type);
+	break;
       } // switch
     } // while XEvent-loopen
-    usleep(1000000L);
+    usleep(200000L);  /* 200ms */
     
   }  // While(1) 
+  pop3Quit(pc);
 }
 
 
@@ -457,7 +504,7 @@ int  readConfigFile(char *filename) {
     snprintf(config_file, sizeof(config_file), "%s/.wmmultipop3rc", getenv("HOME"));
     printf("-Config file does not exit : %s\n",config_file);
     printf("+Trying to create new config file.\n");
-    if((fp = fopen(config_file,"w")) == 0){
+    if ((fp = fopen(config_file,"w")) == 0) {
       printf("-Error creating new config file\n");
       return -1;
     }
@@ -481,11 +528,20 @@ int  readConfigFile(char *filename) {
     fprintf(fp, "#username          < pop3 login name  >\n");
     fprintf(fp, "#password          < pop3 password    >\n");
     fprintf(fp, "#mailserverid      < mail server id string (3 chars) >\n");
+    fprintf(fp, "# To use a local mailbox, set the file name as the pop server.\n");
+    fprintf(fp, "# The port number, username and password are ignored.\n");
+    fprintf(fp, "#[Mailserver 3]\n");
+    fprintf(fp, "#popserver         /var/spool/mail/< your_user_name >\n");
+    fprintf(fp, "#port              110    # default port\n");
+    fprintf(fp, "#username          ignored\n");
+    fprintf(fp, "#password          ignored\n");
+    fprintf(fp, "#mailserverid      < mailbox id string (3 chars) >\n");
     fprintf(fp, "# Add other mail servers here if you want\n");
     printf("+New config file created : ~/.wmmultipop3rc\n\n");
     printf("+ ~/.wmmultipop3rc must be configured before running wmmultipop3.\n");
     fclose(fp);
     return -1;
+    conf.mail_arrived_cmd[0]=0;
   }
 
   /*
@@ -546,6 +602,14 @@ int  readConfigFile(char *filename) {
 	}
 	strncpy(conf.right_click_cmd, ptr, sizeof(conf.right_click_cmd));
       }
+      if ( !strcmp(ptr, "mail_arrived_cmd") ) {
+	ptr = strtok( 0, "\n");
+	if ( strlen(ptr)>(STR_LENGTH-1) )  {
+	  printf("Error in mail_arrived_cmd string: maximum size is %d\n",STR_LENGTH-1);
+	  return -1;
+	}
+	strcpy(conf.mail_arrived_cmd, ptr);
+      }
       if ( !strcmp(ptr, "run_command") ) {
 	ptr = strtok( 0, "\n");
 	if ( ptr == 0 ) {
@@ -634,7 +698,8 @@ void printConfig(void) {
   int i;
   printf("Mailclient:               %s\n", conf.mailclient);
   printf("Right click               %s\n", conf.right_click_cmd);
-  printf("Middle click               %s\n", conf.middle_click_cmd);
+  printf("Middle click              %s\n", conf.middle_click_cmd);
+  printf("Mail arrived command      %s\n", conf.mail_arrived_cmd);
   printf("Autochecking:             %d\n", conf.autoChecking);
   printf("Mailcheckdelay:           %d\n", conf.mailCheckDelay);
   printf("Number of mailboxes:      %d\n", conf.nMailBoxes);
@@ -663,33 +728,27 @@ void BlitString(char *name, int x, int y)
   int		k;
 
   k = x;
-  for (i=0; name[i]; i++)
-    {
-      c = toupper(name[i]); 
-      if (c >= 'A' && c <= 'Z')
-        {   /* its a letter */
-	  c -= 'A';
-	  copyXPMArea(c * 6, 74, 6, 8, k, y);
-	  k += 6;
-        }
-      else if( c == ' ' ){
-	c = 14;
-	copyXPMArea(73, 64, 6, 8, k, y);
-	k += 6;
-      }
-      else if( c == '/' ){
-	c = 14;
-	copyXPMArea(68, 64, 6, 8, k, y);
-	k += 6;
-      }
-      else
-        {   /* its a number or symbol */
-	  c -= '0';
-	  copyXPMArea(c * 6, 64, 6, 8, k, y);
-	  k += 6;
-	}
+  for (i=0; name[i]; i++) {
+    c = toupper(name[i]); 
+    if (c >= 'A' && c <= 'Z') {   
+      /* its a letter */
+      c -= 'A';
+      copyXPMArea(c * 6, 74, 6, 8, k, y);
+      k += 6;
+    } else if( c == ' ' ) {
+      c = 14;
+      copyXPMArea(73, 64, 6, 8, k, y);
+      k += 6;
+    } else if( c == '/' ) {
+      c = 14;
+      copyXPMArea(68, 64, 6, 8, k, y);
+      k += 6;
+    } else {   /* its a number or symbol */
+      c -= '0';
+      copyXPMArea(c * 6, 64, 6, 8, k, y);
+      k += 6;
     }
-
+  }
 }
 
 
